@@ -3,6 +3,8 @@
                              -------------------
     begin                : Sat Oct 11 2003
     copyright            : (C) 2003 by Stanislav Karchebny
+                           (C) 2005 Ian Monroe
+                           (C) 2005 Seb Ruiz
     email                : berkus@users.sf.net
  ***************************************************************************/
 
@@ -67,6 +69,7 @@ k_dcop:
    virtual QString title() = 0;                             ///< Return the title of the currently playing track.
    virtual QString totalTime() = 0;                         ///< Return the total length of the currently playing track ([h:]mm:ss format).
    virtual QString track() = 0;                             ///< Return the track number.
+   virtual QString type() = 0;                             ///< Return the file type.
    virtual QString year() = 0;                              ///< Return the year of the currently playing track.
    virtual void configEqualizer() = 0;                      ///< Toggle equalizer config dialog.
    virtual void enableDynamicMode(bool enable) = 0;         ///< Switch Dynamic Mode on or off.
@@ -129,6 +132,26 @@ k_dcop:
    virtual void togglePlaylist() = 0;                       ///< Toggle the Playlist-window.
 };
 
+class AmarokPlaylistBrowserInterface : virtual public DCOPObject
+{
+   K_DCOP
+
+k_dcop:
+   virtual void addPodcast(const QString &) = 0;            ///< Add a podcast entry to the playlist browser.
+   virtual void scanPodcasts() = 0;                         ///< Scan all podcasts for updates.
+   virtual void addPlaylist(const QString &) = 0;           ///< Add a playlist to the playlist browser.
+};
+
+class AmarokContextBrowserInterface : virtual public DCOPObject
+{
+   K_DCOP
+
+k_dcop:
+   virtual void showHome() = 0;                             ///< Show the home tab in the context browser.
+   virtual void showCurrentTrack() = 0;                     ///< Show the current track in the context browser.
+   virtual void showLyrics() = 0;                           ///< Show the lyrics tab in the context browser.
+   virtual void showWiki() = 0;                             ///< Show the wikipedia tab in the context browser.
+};
 
 class AmarokCollectionInterface : virtual public DCOPObject
 {
@@ -142,6 +165,7 @@ k_dcop:
    virtual int totalTracks() = 0;                       ///< Returns the total of tracks in the collection.
    virtual QStringList query(const QString& sql) = 0;       ///< Queries the database via SQL.
    virtual QStringList similarArtists( int artists ) = 0;       ///< Return similar artists of the current tracks, limit to int artists.
+   virtual void migrateFile( const QString &oldURL, const QString &newURL ) = 0; ///<Move a file in the collection, keeping stats intact.
    virtual void scanCollection() = 0;                       ///< Scan the collection.
    virtual void scanCollectionChanges() = 0;                ///< Scan the collection for changes only.
 };
@@ -157,6 +181,9 @@ k_dcop:
    virtual QStringList listRunningScripts() = 0;            ///< Returns a list of all currently running scripts.
    virtual void addCustomMenuItem(QString submenu, QString itemTitle ) = 0;  ///< Enables and sets custom menu item title.
    virtual void removeCustomMenuItem(QString submenu, QString itemTitle ) = 0;  ///< Removes the custom menu item.
+   virtual QString readConfig(const QString& key) = 0; ///< returns a AmarokConfig configuration entry value from the given key
+   virtual QStringList readListConfig(const QString& key) = 0; ///< AmarokConfig lists must use this function
+
 };
 
 
