@@ -20,6 +20,7 @@
 #include "mediabrowser.h"
 #include "metabundle.h"
 #include "playlist.h"       //insertMedia()
+#include "playlistbrowser.h"
 #include "statusbar.h"
 #include "tagdialog.h"
 
@@ -177,25 +178,43 @@ CollectionBrowser::refreshInfo()
     if( !m_infoA || !m_infoB )
         return;
 
-    QStringList a = CollectionDB::instance()->query( "SELECT COUNT( url ) FROM tags;" );
+    int q_cat1 = m_view->m_cat1, q_cat2 = m_view->m_cat2, q_cat3 = m_view->m_cat3;
+    if( q_cat1 == IdVisYearAlbum )
+        q_cat1 = IdAlbum;
+    if( q_cat2 == IdVisYearAlbum )
+        q_cat2 = IdAlbum;
+    if( q_cat3 == IdVisYearAlbum )
+        q_cat3 = IdAlbum;
+
+    QueryBuilder qb;
+    qb.addReturnFunctionValue( QueryBuilder::funcCount, QueryBuilder::tabSong, QueryBuilder::valURL );
+    qb.setGoogleFilter( q_cat1 | q_cat2 | q_cat3 | QueryBuilder::tabSong, m_view->m_filter );
+    qb.setOptions( QueryBuilder::optRemoveDuplicates );
+    QStringList a = qb.run();
 
     QStringList b;
     QString descriptor;
+    qb.clear();
+    qb.setGoogleFilter( q_cat1 | q_cat2 | q_cat3 | QueryBuilder::tabSong, m_view->m_filter );
+    qb.setOptions( QueryBuilder::optRemoveDuplicates );
     if( m_view )
     {
-        switch( m_view->m_cat1 )
+        switch( q_cat1 )
         {
             case IdArtist:
-                b = CollectionDB::instance()->query( "SELECT COUNT( id ) FROM artist;" );
-                descriptor = i18n( "1 Artist", "%n Artists", b[0].toInt() );
+                qb.addReturnValue( QueryBuilder::tabArtist, QueryBuilder::valID );
+                b = qb.run();
+                descriptor = i18n( "1 Artist", "%n Artists", b.count() );
                 break;
             case IdAlbum:
-                b = CollectionDB::instance()->query( "SELECT COUNT( id ) FROM album;" );
-                descriptor = i18n( "1 Album", "%n Albums", b[0].toInt() );
+                qb.addReturnValue( QueryBuilder::tabAlbum, QueryBuilder::valID );
+                b = qb.run();
+                descriptor = i18n( "1 Album", "%n Albums", b.count() );
                 break;
             case IdGenre:
-                b = CollectionDB::instance()->query( "SELECT COUNT( id ) FROM genre;" );
-                descriptor = i18n( "1 Genre", "%n Genres", b[0].toInt() );
+                qb.addReturnValue( QueryBuilder::tabGenre, QueryBuilder::valID );
+                b = qb.run();
+                descriptor = i18n( "1 Genre", "%n Genres", b.count() );
                 break;
         }
     }
@@ -207,7 +226,7 @@ CollectionBrowser::refreshInfo()
     else
     {
         m_infoA->setText( i18n("1 Track","%n Tracks", a[0].toInt()) );
-	m_infoA->show();
+        m_infoA->show();
     }
     if( b.isEmpty() )
     {
@@ -215,7 +234,7 @@ CollectionBrowser::refreshInfo()
     }
     else
     {
-	m_infoB->setText( descriptor );
+        m_infoB->setText( descriptor );
         m_infoB->show();
     }
 }
@@ -340,6 +359,7 @@ CollectionView::CollectionView( CollectionBrowser* parent )
 
     setSelectionMode( QListView::Extended );
     setItemsMovable( false );
+    setSorting( 0 );
     setShowSortIndicator( true );
     setAcceptDrops( false );
     setAllColumnsShowFocus( true );
@@ -350,6 +370,7 @@ CollectionView::CollectionView( CollectionBrowser* parent )
         m_cat2 = config->readNumEntry( "Category2", CollectionBrowser::IdAlbum );
         m_cat3 = config->readNumEntry( "Category3", CollectionBrowser::IdNone );
         m_viewMode = config->readNumEntry( "ViewMode", modeTreeView );
+        updateTrackDepth();
     //</READ CONFIG>
      KActionCollection* ac = new KActionCollection( this );
      KStdAction::selectAll( this, SLOT( selectAll() ), ac, "collectionview_select_all" );
@@ -470,7 +491,7 @@ CollectionView::renderView()  //SLOT
         if( q_cat3 != CollectionBrowser::IdNone )
             qb.sortBy( q_cat3, QueryBuilder::valName );
 
-        setQBFilters( qb, m_filter, q_cat1 | q_cat2 | q_cat3 | QueryBuilder::tabSong );
+        qb.setGoogleFilter( q_cat1 | q_cat2 | q_cat3 | QueryBuilder::tabSong, m_filter );
         qb.setOptions( QueryBuilder::optRemoveDuplicates );
 
         values = qb.run();
@@ -481,7 +502,7 @@ CollectionView::renderView()  //SLOT
             if( values[i].stripWhiteSpace().isEmpty() )
                 values[i] = i18n( "Unknown" );
 
-            Item* item = new Item( this );
+            CollectionItem* item = new CollectionItem( this );
             item->setDragEnabled( true );
             item->setDropEnabled( false );
 
@@ -516,7 +537,7 @@ CollectionView::renderView()  //SLOT
         if( VisYearAlbum == 1 )
             qb.addReturnValue( QueryBuilder::tabYear, QueryBuilder::valName );
 
-        setQBFilters( qb, m_filter, q_cat1 | q_cat2 | q_cat3 | QueryBuilder::tabSong );
+        qb.setGoogleFilter( q_cat1 | q_cat2 | q_cat3 | QueryBuilder::tabSong, m_filter );
 
         if( VisYearAlbum == 1 )
             qb.sortBy( QueryBuilder::tabYear, QueryBuilder::valName );
@@ -531,20 +552,21 @@ CollectionView::renderView()  //SLOT
 
         //add items to the view
 
-        // if year - artist is cat 1 rebuild the list
-        if( VisYearAlbum == 1 )
-        {
-            QStringList tmpvalues;
-            for( unsigned int i=0; i < values.count()-1 ; i += 2 )
-            {
-                tmpvalues += ( values[i+1].isEmpty() ? "?" : values[i+1]) +
-                               i18n( " - " ) +
-                             ( values[i].isEmpty() ? i18n( "Unknown" ) : values[i] );
-            }
-            values = tmpvalues;
-        }
-
         if( values.count() )
+        {
+            // if year - artist is cat 1 rebuild the list
+            if( VisYearAlbum == 1 )
+            {
+                QStringList tmpvalues;
+                for( uint i = 0; i < values.count(); i += 2 )
+                {
+                    tmpvalues += ( values[i+1].isEmpty() ? "?" : values[i+1]) +
+                    i18n( " - " ) +
+                    ( values[i].isEmpty() ? i18n( "Unknown" ) : values[i] );
+                }
+                values = tmpvalues;
+            }
+
             for ( QStringList::Iterator it = values.fromLast(), begin = values.begin(); true; --it )
             {
                 if ( (*it).stripWhiteSpace().isEmpty() )
@@ -553,7 +575,7 @@ CollectionView::renderView()  //SLOT
                 if ( (*it).startsWith( "the ", false ) )
                     manipulateThe( *it, true );
 
-                KListViewItem* item = new KListViewItem( this );
+                KListViewItem* item = new CollectionItem( this, m_cat1 );
                 item->setExpandable( true );
                 item->setDragEnabled( true );
                 item->setDropEnabled( false );
@@ -563,33 +585,37 @@ CollectionView::renderView()  //SLOT
                 if ( it == begin )
                     break;
             }
+        }
 
         //check if we need to add a Various Artists node
         if ( q_cat1 == QueryBuilder::tabArtist )
         {
             qb.clear();
             qb.addReturnValue( q_cat1, QueryBuilder::valName );
-            setQBFilters( qb, m_filter, q_cat1 | q_cat2 | q_cat3 | QueryBuilder::tabSong );
+            qb.setGoogleFilter( q_cat1 | q_cat2 | q_cat3 | QueryBuilder::tabSong, m_filter );
             qb.setOptions( QueryBuilder::optOnlyCompilations | QueryBuilder::optRemoveDuplicates );
             qb.setLimit( 0, 1 );
             values = qb.run();
 
             if ( values.count() )
             {
-                KListViewItem* item = new KListViewItem( this, lastItem() );
+                KListViewItem* item = new CollectionItem( this, m_cat1 );
                 item->setExpandable( true );
                 item->setDragEnabled( true );
                 item->setDropEnabled( false );
-                item->setText( 0, i18n( "Various Artists" ) );
+                    item->setText( 0, i18n( "Various Artists" ) );
                 item->setPixmap( 0, pixmap );
             }
         }
 
-        // we sort because of the items which should begin with 'the' are at the 't' location
-        setSorting( 0 );
-        sort();
-        // we then disable it, as we don't want the children being sorted later as well
-        setSorting( -1 );
+        int count = childCount();
+        QListViewItem *item = firstChild();
+        while( count == 1 && item && item->isExpandable() )
+        {
+            item->setOpen( true );
+            count = item->childCount();
+            item = item->firstChild();
+        }
     }
     restoreView();
 }
@@ -610,8 +636,8 @@ CollectionView::setupDirs()  //SLOT
     dialog.setMainWidget( setup );
     dialog.showButtonApply( false );
     dialog.adjustSize();
-    // Make the dialog a bit wider, default is too narrow to be useful
-    dialog.resize( dialog.width() + 50, dialog.height() );
+    // Make the dialog a bit bigger, default is too small to be useful
+    dialog.resize( dialog.width() + 50, dialog.height() + 150 );
 
     if ( dialog.exec() != QDialog::Rejected )
     {
@@ -634,15 +660,11 @@ CollectionView::scanStarted() // SLOT
 
 
 void
-CollectionView::scanDone( bool changed ) //SLOT
+CollectionView::scanDone( bool /*changed*/ ) //SLOT
 {
     DEBUG_BLOCK
 
-    if ( changed )
-    {
-        renderView();
-        m_parent->refreshInfo();
-    }
+    renderView();
     m_parent->m_scanAction->setEnabled( !AmarokConfig::monitorChanges() );
 }
 
@@ -658,6 +680,13 @@ CollectionView::slotExpand( QListViewItem* item )  //SLOT
 
     QueryBuilder qb;
     bool c = false;
+    bool SortbyTrackFirst=false;
+
+    //Sort by track number first if album is in one of the categories, otherwise by track name first
+    if (m_cat1 == CollectionBrowser::IdAlbum ||
+        m_cat2 == CollectionBrowser::IdAlbum ||
+        m_cat3 == CollectionBrowser::IdAlbum)
+            SortbyTrackFirst = true;
 
     // initalization for year - album mode
     QString tmptext;
@@ -669,6 +698,7 @@ CollectionView::slotExpand( QListViewItem* item )  //SLOT
         m_cat2 == CollectionBrowser::IdVisYearAlbum ||
         m_cat3 == CollectionBrowser::IdVisYearAlbum )
     {
+        SortbyTrackFirst = true;
         if( m_cat1 == CollectionBrowser::IdVisYearAlbum )
         {
             VisYearAlbum = 1;
@@ -721,7 +751,12 @@ CollectionView::slotExpand( QListViewItem* item )  //SLOT
                 qb.addReturnValue( q_cat2, QueryBuilder::valTitle );
                 qb.addReturnValue( q_cat2, QueryBuilder::valURL );
                 if ( c ) qb.addReturnValue( QueryBuilder::tabArtist, QueryBuilder::valName );
-                qb.sortBy( q_cat2, QueryBuilder::valTrack );
+
+                if (SortbyTrackFirst)
+                    qb.sortBy( q_cat2, QueryBuilder::valTrack );
+                qb.sortBy( q_cat2, QueryBuilder::valTitle );
+                if (!SortbyTrackFirst)
+                    qb.sortBy( q_cat2, QueryBuilder::valTrack );
                 qb.sortBy( q_cat2, QueryBuilder::valURL );
             }
             else
@@ -791,7 +826,11 @@ CollectionView::slotExpand( QListViewItem* item )  //SLOT
                 qb.addReturnValue( q_cat3, QueryBuilder::valURL );
                 if ( c ) qb.addReturnValue( QueryBuilder::tabArtist, QueryBuilder::valName );
 
-                qb.sortBy( q_cat3, QueryBuilder::valTrack );
+                if (SortbyTrackFirst)
+                    qb.sortBy( q_cat3, QueryBuilder::valTrack );
+                qb.sortBy( q_cat3, QueryBuilder::valTitle );
+                if (!SortbyTrackFirst)
+                    qb.sortBy( q_cat3, QueryBuilder::valTrack );
                 qb.sortBy( q_cat3, QueryBuilder::valURL );
             }
             else
@@ -887,14 +926,18 @@ CollectionView::slotExpand( QListViewItem* item )  //SLOT
             if( c )
                 qb.addReturnValue( QueryBuilder::tabArtist, QueryBuilder::valName );
 
-            qb.sortBy( QueryBuilder::tabSong, QueryBuilder::valTrack );
+            if (SortbyTrackFirst)
+                qb.sortBy( QueryBuilder::tabSong, QueryBuilder::valTrack );
+            qb.sortBy( QueryBuilder::tabSong, QueryBuilder::valTitle );
+            if (!SortbyTrackFirst)
+                qb.sortBy( QueryBuilder::tabSong, QueryBuilder::valTrack );
             qb.sortBy( QueryBuilder::tabSong, QueryBuilder::valURL );
 
             category = CollectionBrowser::IdNone;
             break;
     }
 
-    setQBFilters( qb, m_filter, q_cat1 | q_cat2 | q_cat3 | QueryBuilder::tabSong );
+    qb.setGoogleFilter( q_cat1 | q_cat2 | q_cat3 | QueryBuilder::tabSong, m_filter );
     qb.setOptions( QueryBuilder::optRemoveDuplicates );
     values = qb.run();
     int countReturnValues = qb.countReturnValues();
@@ -931,7 +974,7 @@ CollectionView::slotExpand( QListViewItem* item )  //SLOT
             text = values[ i + 2 ].stripWhiteSpace().isEmpty() ? i18n( "Unknown" ) : values[ i + 2 ] + i18n(" - ");
         text += values[ i ].stripWhiteSpace().isEmpty() ? i18n( "Unknown" ) : values[ i ];
 
-        Item* child = new Item( item );
+        CollectionItem* child = new CollectionItem( item, category );
         child->setDragEnabled( true );
         child->setDropEnabled( false );
         child->setText( 0, text );
@@ -1030,7 +1073,7 @@ CollectionView::cat1Menu( int id, bool rerender )  //SLOT
         m_parent->m_cat3Menu->setItemChecked( CollectionBrowser::IdNone, true );
         m_cat3 = CollectionBrowser::IdNone;
     }
-
+    updateTrackDepth();
     if ( rerender )
         renderView();
 }
@@ -1057,7 +1100,7 @@ CollectionView::cat2Menu( int id, bool rerender )  //SLOT
         m_parent->m_cat3Menu->setItemChecked( id, false );
         enableCat3Menu( false );
     }
-
+    updateTrackDepth();
     if ( rerender )
         renderView();
 }
@@ -1070,7 +1113,7 @@ CollectionView::cat3Menu( int id, bool rerender )  //SLOT
     m_cat3 = id;
     m_parent->m_cat3Menu->setItemChecked( m_cat3, true );
     updateColumnHeader();
-
+    updateTrackDepth();
     if ( rerender )
         renderView();
 }
@@ -1090,6 +1133,7 @@ CollectionView::enableCat3Menu( bool enable )
         m_parent->m_cat3Menu->setItemChecked( CollectionBrowser::IdNone, true );
         m_cat3 = CollectionBrowser::IdNone;
     }
+    updateTrackDepth();
 }
 
 
@@ -1098,12 +1142,11 @@ CollectionView::invokeItem( QListViewItem* item ) //SLOT
 {
     if ( !item )
         return;
-
+    //append and prevent doubles in playlist
     if( item->isExpandable() )
-        item->setOpen( !item->isOpen() );
+        Playlist::instance()->insertMedia( listSelected(), Playlist::Unique | Playlist::Append );
     else
-        //direct play & prevent doubles in playlist
-        Playlist::instance()->insertMedia( static_cast<Item*>( item )->url(), Playlist::Unique | Playlist::DirectPlay );
+        Playlist::instance()->insertMedia( static_cast<CollectionItem*>( item )->url(), Playlist::Unique | Playlist::Append );
 
 }
 
@@ -1129,23 +1172,26 @@ CollectionView::rmbPressed( QListViewItem* item, const QPoint& point, int ) //SL
         }
 
         #ifdef AMAZON_SUPPORT
-        enum Actions { APPEND, MAKE, QUEUE, MEDIA_DEVICE, BURN_ARTIST, BURN_ALBUM,
-                       BURN_DATACD, BURN_AUDIOCD, COVER, INFO,
+        enum Actions { APPEND, QUEUE, MAKE, SAVE, MEDIA_DEVICE, BURN_ARTIST, BURN_ALBUM, BURN_CD, COVER, INFO,
                        COMPILATION_SET, COMPILATION_UNSET  };
         #else
-        enum Actions { APPEND, MAKE, QUEUE, BURN_ARTIST, BURN_ALBUM,
-                       BURN_DATACD, BURN_AUDIOCD, INFO,
-                       COMPILATION_SET, COMPILATION_UNSET, MEDIA_DEVICE };
+        enum Actions { APPEND, QUEUE, MAKE, SAVE, MEDIA_DEVICE, BURN_ARTIST, BURN_ALBUM, BURN_CD, INFO,
+                       COMPILATION_SET, COMPILATION_UNSET  };
         #endif
         KURL::List selection = listSelected();
+        menu.insertItem( SmallIconSet( "fileopen" ), i18n( "&Load" ), MAKE );
         menu.insertItem( SmallIconSet( "1downarrow" ), i18n( "&Append to Playlist" ), APPEND );
         menu.insertItem( SmallIconSet( "2rightarrow" ), selection.count() == 1 ? i18n( "&Queue Track" )
             : i18n( "&Queue Tracks" ), QUEUE );
-        menu.insertItem( SmallIconSet( "player_playlist_2" ), i18n( "&Make Playlist" ), MAKE );
-        menu.insertItem( SmallIconSet( "usbpendrive_unmount" ), i18n( "Add to Media Device &Transfer Queue" ), MEDIA_DEVICE );
+
+        if( selection.count() > 1 || item->isExpandable() )
+            menu.insertItem( SmallIconSet( "filesave" ), i18n( "&Save as Playlist..." ), SAVE );
 
         menu.insertSeparator();
 
+        if( MediaDevice::instance()->isConnected() )
+            menu.insertItem( SmallIconSet( "usbpendrive_unmount" ), i18n( "Add to Media Device &Transfer Queue" ), MEDIA_DEVICE );
+
         if( cat == CollectionBrowser::IdArtist )
         {
             menu.insertItem( SmallIconSet( "cdrom_unmount" ), i18n("Burn All Tracks by This Artist"), BURN_ARTIST );
@@ -1158,17 +1204,15 @@ CollectionView::rmbPressed( QListViewItem* item, const QPoint& point, int ) //SL
         }
         else if( !item->isExpandable() )
         {
-            menu.insertItem( SmallIconSet( "cdrom_unmount" ), i18n("Burn to CD as Data"), BURN_DATACD );
-            menu.setItemEnabled( BURN_DATACD, K3bExporter::isAvailable() );
-            menu.insertItem( SmallIconSet( "cdaudio_unmount" ), i18n("Burn to CD as Audio"), BURN_AUDIOCD );
-            menu.setItemEnabled( BURN_AUDIOCD, K3bExporter::isAvailable() );
+            menu.insertItem( SmallIconSet( "cdrom_unmount" ), i18n("Burn to CD"), BURN_CD );
+            menu.setItemEnabled( BURN_CD, K3bExporter::isAvailable() );
         }
 
         menu.insertSeparator();
 
         #ifdef AMAZON_SUPPORT
-        menu.insertItem( SmallIconSet( "www" ), i18n( "&Fetch Cover Images" ), this, SLOT( fetchCover() ), 0, COVER );
-        menu.setItemEnabled(COVER, cat == CollectionBrowser::IdAlbum );
+        menu.insertItem( SmallIconSet( "www" ), i18n( "&Fetch Cover Image" ), this, SLOT( fetchCover() ), 0, COVER );
+        menu.setItemEnabled(COVER, cat == CollectionBrowser::IdAlbum || cat == CollectionBrowser::IdVisYearAlbum );
         #endif
         menu.insertItem( SmallIconSet( "info" )
             , i18n( "Edit Track &Information...",  "Edit &Information for %n Tracks...", selection.count())
@@ -1186,13 +1230,16 @@ CollectionView::rmbPressed( QListViewItem* item, const QPoint& point, int ) //SL
             case APPEND:
                 Playlist::instance()->insertMedia( selection, Playlist::Append );
                 break;
-           case MAKE:
+            case MAKE:
                 Playlist::instance()->insertMedia( selection, Playlist::Replace );
                 break;
+            case SAVE:
+                playlistFromURLs( selection );
+                break;
             case QUEUE:
                 Playlist::instance()->insertMedia( selection, Playlist::Queue );
                 break;
-             case MEDIA_DEVICE:
+            case MEDIA_DEVICE:
                 MediaDevice::instance()->addURLs( selection );
                 break;
             case BURN_ARTIST:
@@ -1206,11 +1253,8 @@ CollectionView::rmbPressed( QListViewItem* item, const QPoint& point, int ) //SL
                 else
                     K3bExporter::instance()->exportAlbum( item->text(0) );
                 break;
-            case BURN_DATACD:
-                K3bExporter::instance()->exportTracks( selection, K3bExporter::DataCD );
-                break;
-            case BURN_AUDIOCD:
-                K3bExporter::instance()->exportTracks( selection, K3bExporter::AudioCD );
+            case BURN_CD:
+                K3bExporter::instance()->exportTracks( selection );
                 break;
             case COMPILATION_SET:
                 CollectionDB::instance()->setCompilation( item->text(0), true );
@@ -1244,7 +1288,6 @@ CollectionView::setViewMode( int mode, bool rerender )
         setResizeMode( QListView::LastColumn );
         setRootIsDecorated( true );
         setFullWidth( true );
-        setSorting( -1 ); //disables sorting
     }
     else
     {
@@ -1255,7 +1298,6 @@ CollectionView::setViewMode( int mode, bool rerender )
 
         setResizeMode( QListView::AllColumns );
         setRootIsDecorated( false );
-        setSorting( 0 );
     }
 
     m_viewMode = mode;
@@ -1268,10 +1310,30 @@ void
 CollectionView::fetchCover() //SLOT
 {
     #ifdef AMAZON_SUPPORT
-    Item* item = static_cast<Item*>( currentItem() );
+    CollectionItem* item = static_cast<CollectionItem*>( currentItem() );
     if ( !item ) return;
 
+    int cat = 0;
+    switch ( item->depth() )
+    {
+        case 0:
+            cat = m_cat1;
+            break;
+        case 1:
+            cat = m_cat2;
+            break;
+        case 2:
+            cat = m_cat3;
+            break;
+    }
+
     QString album = item->text(0);
+    if( cat == CollectionBrowser::IdVisYearAlbum )
+    {
+        // we can't use findRev since an album may have " - " within it.
+        QString sep = i18n(" - ");
+        album = album.right( album.length() - sep.length() - album.find( sep ) );
+    }
 
     // find the first artist's name
     QStringList values =
@@ -1393,7 +1455,7 @@ CollectionView::listSelected()
     {
         for ( item = firstChild(); item; item = item->nextSibling() )
             if ( item->isSelected() )
-                list << static_cast<Item*>( item ) ->url();
+                list << static_cast<CollectionItem*>( item ) ->url();
 
         return list;
     }
@@ -1420,7 +1482,6 @@ CollectionView::listSelected()
                 {
                     QString year = tmptext.left( tmptext.find( i18n(" - ") ) );
                     yearAlbumCalc( year, tmptext );
-                    tmptext = tmptext.right(tmptext.find(i18n(" - ")));
                     qb.addMatch( QueryBuilder::tabYear, year );
                     qb.addMatch( q_cat1, tmptext );
                 }
@@ -1430,7 +1491,7 @@ CollectionView::listSelected()
             else
                 qb.setOptions( QueryBuilder::optOnlyCompilations );
 
-            setQBFilters( qb, m_filter, q_cat1 | q_cat2 | q_cat3 | QueryBuilder::tabSong );
+            qb.setGoogleFilter( q_cat1 | q_cat2 | q_cat3 | QueryBuilder::tabSong, m_filter );
 
             if( VisYearAlbum == 1 )
                 qb.sortBy( QueryBuilder::tabYear, QueryBuilder::valName);
@@ -1469,7 +1530,7 @@ CollectionView::listSelected()
         for ( item = firstChild(); item; item = item->nextSibling() )
             for ( QListViewItem* child = item->firstChild(); child; child = child->nextSibling() )
                 if ( child->isSelected() && !child->parent()->isSelected() )
-                    list << static_cast<Item*>( child ) ->url();
+                    list << static_cast<CollectionItem*>( child ) ->url();
     }
     else {
         for ( item = firstChild(); item; item = item->nextSibling() )
@@ -1495,7 +1556,6 @@ CollectionView::listSelected()
                         {
                             QString year = tmptext.left( tmptext.find( i18n(" - ") ) );
                             yearAlbumCalc( year, tmptext );
-                            tmptext = tmptext.right(tmptext.find(i18n(" - ")));
                             qb.addMatch( QueryBuilder::tabYear, year );
                             qb.addMatch( q_cat1, tmptext );
                         }
@@ -1518,14 +1578,13 @@ CollectionView::listSelected()
                     {
                         QString year = tmptext.left( tmptext.find( i18n(" - ") ) );
                         yearAlbumCalc( year, tmptext );
-                        tmptext = tmptext.right(tmptext.find(i18n(" - ")));
                         qb.addMatch( QueryBuilder::tabYear, year );
                         qb.addMatch( q_cat2, tmptext );
                     }
                     else
                         qb.addMatches( q_cat2, matches );
 
-                    setQBFilters( qb, m_filter, q_cat1 | q_cat2 | q_cat3 | QueryBuilder::tabSong );
+                    qb.setGoogleFilter( q_cat1 | q_cat2 | q_cat3 | QueryBuilder::tabSong, m_filter );
 
                     if( VisYearAlbum == 1 )
                         qb.sortBy( QueryBuilder::tabYear, QueryBuilder::valName);
@@ -1564,7 +1623,7 @@ CollectionView::listSelected()
         for ( QListViewItem* child = item->firstChild(); child; child = child->nextSibling() )
             for ( QListViewItem* grandChild = child->firstChild(); grandChild; grandChild = grandChild->nextSibling() )
                 if ( grandChild->isSelected() && !grandChild->isExpandable() && !child->parent()->isSelected() && !child->isSelected() )
-                    list << static_cast<Item*>( grandChild ) ->url();
+                    list << static_cast<CollectionItem*>( grandChild ) ->url();
 
     //category 3
     if ( m_cat3 == CollectionBrowser::IdNone )
@@ -1574,7 +1633,7 @@ CollectionView::listSelected()
                 for ( QListViewItem* grandChild = child->firstChild(); grandChild; grandChild = grandChild->nextSibling() )
                     for ( QListViewItem* grandChild2 = grandChild->firstChild(); grandChild2; grandChild2 = grandChild2->nextSibling() )
                         if ( grandChild2->isSelected() && !child->parent()->isSelected() && !child->isSelected() && !grandChild->isSelected() )
-                            list << static_cast<Item*>( grandChild2 ) ->url();
+                            list << static_cast<CollectionItem*>( grandChild2 ) ->url();
     }
     else {
         for ( item = firstChild(); item; item = item->nextSibling() )
@@ -1601,7 +1660,6 @@ CollectionView::listSelected()
                             {
                                 QString year = tmptext.left( tmptext.find( i18n(" - ") ) );
                                 yearAlbumCalc( year, tmptext );
-                                tmptext = tmptext.right(tmptext.find(i18n(" - ")));
                                 qb.addMatch( QueryBuilder::tabYear, year );
                                 qb.addMatch( q_cat1, tmptext );
                             }
@@ -1623,7 +1681,6 @@ CollectionView::listSelected()
                         {
                             QString year = tmptext.left( tmptext.find( i18n(" - ") ) );
                             yearAlbumCalc( year, tmptext );
-                            tmptext = tmptext.right(tmptext.find(i18n(" - ")));
                             qb.addMatch( QueryBuilder::tabYear, year );
                             qb.addMatch( q_cat2, tmptext );
                         }
@@ -1643,7 +1700,6 @@ CollectionView::listSelected()
                         {
                             QString year = tmptext.left( tmptext.find( i18n(" - ") ) );
                             yearAlbumCalc( year, tmptext );
-                            tmptext = tmptext.right(tmptext.find(i18n(" - ")));
                             qb.addMatch( QueryBuilder::tabYear, year );
                             qb.addMatch( q_cat3, tmptext );
                         }
@@ -1653,7 +1709,7 @@ CollectionView::listSelected()
 
                             qb.addMatches( q_cat3, matches );
                         }
-                        setQBFilters( qb, m_filter, q_cat1 | q_cat2 | q_cat3 | QueryBuilder::tabSong );
+                        qb.setGoogleFilter( q_cat1 | q_cat2 | q_cat3 | QueryBuilder::tabSong, m_filter );
 
                         if( VisYearAlbum == 1 )
                             qb.sortBy( QueryBuilder::tabYear, QueryBuilder::valName);
@@ -1691,11 +1747,42 @@ CollectionView::listSelected()
                 for ( QListViewItem* grandChild = child->firstChild(); grandChild; grandChild = grandChild->nextSibling() )
                     for ( QListViewItem* grandChild2 = grandChild->firstChild(); grandChild2; grandChild2 = grandChild2->nextSibling() )
                         if ( grandChild2->isSelected() && !child->parent()->isSelected() && !child->isSelected() && !grandChild->isSelected() )
-                            list << static_cast<Item*>( grandChild2 ) ->url();
+                            list << static_cast<CollectionItem*>( grandChild2 ) ->url();
 
     return list;
 }
 
+void
+CollectionView::playlistFromURLs( const KURL::List &urls )
+{
+    QString suggestion;
+    typedef QListViewItemIterator It;
+    It it( this, It::Visible | It::Selected );
+    if( (*it) && !(*(++it)) )
+        suggestion = (*It( this, It::Visible | It::Selected ))->text( 0 );
+    else
+        suggestion = i18n( "Untitled" );
+    const QString path = PlaylistDialog::getSaveFileName( suggestion );
+
+    if( path.isEmpty() )
+        return;
+
+    CollectionDB* db = CollectionDB::instance();
+
+    QValueList<QString> titles;
+    QValueList<QString> seconds;
+    for( KURL::List::ConstIterator it = urls.constBegin(), end = urls.constEnd(); it != end; ++it )
+    {
+        const QString query = QString("SELECT title, length FROM tags WHERE url = '%1';")
+                              .arg( db->escapeString( (*it).path() ) ); //no operator->, how suck!
+        QStringList result = db->query( query );
+        titles << result[0];
+        seconds << result[1];
+    }
+
+    if( PlaylistBrowser::savePlaylist( path, urls, titles, seconds ) )
+        PlaylistWindow::self()->showBrowser( "PlaylistBrowser" );
+}
 
 QPixmap
 CollectionView::iconForCategory( const int cat ) const
@@ -1867,188 +1954,6 @@ CollectionView::yearAlbumCalc( QString &year, QString &text )
                        i18n(" - ").length() );
 }
 
-
-void
-CollectionView::setQBFilters( QueryBuilder &qb, QString query, const int &defaults )
-{
-    if( query.contains( "\"" ) % 2 == 1 ) query += "\""; //make an even number of "s
-
-    //something like thingy"bla"stuff -> thingy "bla" stuff
-    bool odd = false;
-    for( int pos = query.find( "\"" );
-         pos >= 0 && pos <= (int)query.length();
-         pos = query.find( "\"", pos + 1 ) )
-    {
-        query = query.insert( odd ? ++pos : pos++, " " );
-        odd = !odd;
-    }
-    query = query.simplifyWhiteSpace();
-
-    int x; //position in string of the end of the next element
-    bool OR = false, minus = false; //whether the next element is to be OR, and/or negated
-    QString tmp, s = "", field = ""; //the current element, a tempstring, and the field: of the next element
-    QStringList tmpl; //list of elements of which at least one has to match (OR)
-    QValueList<QStringList> allof; //list of all the tmpls, of which all have to match
-    while( !query.isEmpty() )  //seperate query into parts which all have to match
-    {
-        if( query.startsWith( " " ) )
-            query = query.mid( 1 ); //cuts off the first character
-        if( query.startsWith( "\"" ) ) //take stuff in "s literally (basically just ends up ignoring spaces)
-        {
-            query = query.mid( 1 );
-            x = query.find( "\"" );
-        }
-        else
-            x = query.find( " " );
-        if( x < 0 )
-            x = query.length();
-        s = query.left( x ); //get the element
-        query = query.mid( x + 1 ); //move on
-
-        if( !field.isEmpty() || ( s != "-" && s != "AND" && s != "OR" &&
-                                  !s.endsWith( ":" ) && !s.endsWith( ":>" ) && !s.endsWith( ":<" ) ) )
-        {
-            if( !OR && !tmpl.isEmpty() ) //add the OR list to the AND list
-            {
-                allof += tmpl;
-                tmpl.clear();
-            }
-            else
-                OR = false;
-            tmp = field + s;
-            if( minus )
-            {
-                tmp = "-" + tmp;
-                minus = false;
-            }
-            tmpl += tmp;
-            tmp = field = "";
-        }
-        else if( s.endsWith( ":" ) || s.endsWith( ":>" ) || s.endsWith( ":<" ) )
-            field = s;
-        else if( s == "OR" )
-            OR = true;
-        else if( s == "-" )
-            minus = true;
-        else
-            OR = false;
-    }
-    if( !tmpl.isEmpty() )
-        allof += tmpl;
-
-    const uint allofcount = allof.count();
-    for( uint i = 0; i < allofcount; ++i ) //check each part for matchiness
-    {
-        qb.beginOR();
-        uint count = allof[i].count();
-        for( uint ii = 0; ii < count; ++ii )
-        {
-            field = QString::null;
-            s = allof[i][ii];
-            bool neg = s.startsWith( "-" );
-            if ( neg )
-                s = s.mid( 1 ); //cut off the -
-            x = s.find( ":" ); //where the field ends and the thing-to-match begins
-            if( x > 0 )
-            {
-                field = s.left( x ).lower();
-                s = s.mid( x + 1 );
-            }
-
-            int mode = QueryBuilder::modeNormal;
-            if( !field.isEmpty() && s.startsWith( ">" ) )
-            {
-                s = s.mid( 1 );
-                mode = QueryBuilder::modeGreater;
-            }
-            else if( !field.isEmpty() && s.startsWith( "<" ) )
-            {
-                s = s.mid( 1 );
-                mode = QueryBuilder::modeLess;
-            }
-
-            int table = -1, value = -1;
-            if( field == "artist" )
-                table = QueryBuilder::tabArtist;
-            else if( field == "album" )
-                table = QueryBuilder::tabAlbum;
-            else if( field == "title" )
-                table = QueryBuilder::tabSong;
-            else if( field == "genre" )
-                table = QueryBuilder::tabGenre;
-            else if( field == "year" )
-                table = QueryBuilder::tabYear;
-            else if( field == "score" )
-            {
-                table = QueryBuilder::tabStats;
-                value = QueryBuilder::valScore;
-            }
-            else if( field == "directory" )
-            {
-                table = QueryBuilder::tabSong;
-                value = QueryBuilder::valDirectory;
-            }
-            else if( field == "length" )
-            {
-                table = QueryBuilder::tabSong;
-                value = QueryBuilder::valLength;
-            }
-            else if( field == "playcount" )
-            {
-                table = QueryBuilder::tabStats;
-                value = QueryBuilder::valPlayCounter;
-            }
-            else if( field == "samplerate" )
-            {
-                table = QueryBuilder::tabSong;
-                value = QueryBuilder::valSamplerate;
-            }
-            else if( field == "track" )
-            {
-                table = QueryBuilder::tabSong;
-                value = QueryBuilder::valTrack;
-            }
-            else if( field == "filename" || field == "url" )
-            {
-                table = QueryBuilder::tabSong;
-                value = QueryBuilder::valURL;
-            }
-            else if( field == "bitrate" )
-            {
-                table = QueryBuilder::tabSong;
-                value = QueryBuilder::valBitrate;
-            }
-            else if( field == "comment" )
-            {
-                table = QueryBuilder::tabSong;
-                value = QueryBuilder::valComment;
-            }
-            else if( field == "lyrics" )
-            {
-                table = QueryBuilder::tabSong;
-                value = QueryBuilder::valLyrics;
-            }
-
-            if( neg )
-            {
-                if( value >= 0 )
-                    qb.excludeFilter( table, value, s, mode );
-                else
-                    qb.excludeFilter( table >= 0 ? table : defaults, s );
-            }
-            else
-            {
-                if( value >= 0 )
-                    qb.addFilter( table, value, s, mode );
-                else
-                    qb.addFilter( table >= 0 ? table : defaults, s );
-            }
-        }
-        qb.endOR();
-    }
-}
-
-
 void
 CollectionView::viewportPaintEvent( QPaintEvent *e )
 {
@@ -2078,6 +1983,24 @@ CollectionView::viewportPaintEvent( QPaintEvent *e )
 }
 
 
+void
+CollectionView::updateTrackDepth() {
+    bool m3 = (m_cat3 == CollectionBrowser::IdNone);
+    bool m2 = (m_cat2 == CollectionBrowser::IdNone);
+    bool m1 = (m_cat1 == CollectionBrowser::IdNone);
+    if ( m3 || m2 || m1) {
+        //The wanted depth, is the lowest IdNone
+        if (m3)
+            m_trackDepth = 2;
+        if (m2)
+            m_trackDepth = 1;
+        if (m1)
+            m_trackDepth = 0;
+    }
+    else // If there's no IdNone, then it's 3
+        m_trackDepth = 3;
+}
+
 void
 CollectionView::viewportResizeEvent( QResizeEvent* )
 {
@@ -2086,4 +2009,60 @@ CollectionView::viewportResizeEvent( QResizeEvent* )
 }
 
 
+//////////////////////////////////////////////////////////////////////////////////////////
+// CLASS CollectionItem
+//////////////////////////////////////////////////////////////////////////////////////////
+int
+CollectionItem::compare( QListViewItem* i, int col, bool /* ascending */) const
+{
+    QString a =    text( col );
+    QString b = i->text( col );
+    int ia, ib;
+
+    switch( m_cat ) {
+        case CollectionBrowser::IdVisYearAlbum:
+            a = a.left( a.find( i18n(" - ") ) );
+            b = b.left( b.find( i18n(" - ") ) );
+            // "?" are the last ones
+            if ( a == "?" )
+                return 1;
+            if ( b == "?" )
+                return -1;
+        //fall through
+        case CollectionBrowser::IdYear:
+            ia = a.toInt();
+            ib = b.toInt();
+            if (ia==ib)
+                return QString::localeAwareCompare( text( col ).lower(), i->text( col ).lower() );
+            if (ia<ib)
+                return 1;
+            else
+                return -1;
+
+        // Various Artists is always the last one
+        case CollectionBrowser::IdArtist:
+            if ( b == i18n("Various Artists") )
+                return -1;
+            if ( a == i18n("Various Artists") )
+                return 1;
+        default:
+        // Unknown is always the first one
+            if ( a == i18n("Unknown") )
+                return -1;
+            if ( b == i18n("Unknown") )
+                return 1;
+    }
+    // No special case, then fall on default
+    return QString::localeAwareCompare( a.lower(), b.lower() );
+}
+
+void
+CollectionItem::sortChildItems ( int column, bool ascending ) {
+    CollectionView* view = static_cast<CollectionView*>( listView() );
+    // Sort only if it's not the tracks
+    if ( depth() + 1 < view->trackDepth())
+        QListViewItem::sortChildItems( column, ascending );
+}
+
+
 #include "collectionbrowser.moc"
