@@ -164,7 +164,7 @@ class PostgresqlConnection : public DbConnection
 class DbConnectionPool : QPtrQueue<DbConnection>
 {
     public:
-        DbConnectionPool();
+        DbConnectionPool( bool temporary );
        ~DbConnectionPool();
 
         const DbConnection::DbConnectionType getDbConnectionType() const { return m_dbConnType; }
@@ -188,6 +188,8 @@ class DbConnectionPool : QPtrQueue<DbConnection>
 
     private:
         static const int POOL_SIZE = 5;
+
+        bool m_isTemporary;
         QSemaphore m_semaphore;
         DbConnection::DbConnectionType m_dbConnType;
         DbConfig *m_dbConfig;
@@ -206,7 +208,8 @@ class CollectionDB : public QObject, public EngineObserver
         void databaseEngineChanged();
 
         void scoreChanged( const QString &url, int score );
-        void coverFetched( const QString &artist, const QString &album );
+        void coverChanged( const QString &artist, const QString &album ); //whenever a cover changes
+        void coverFetched( const QString &artist, const QString &album ); //only when fetching from amazon
         void coverRemoved( const QString &artist, const QString &album );
         void coverFetcherError( const QString &error );
 
@@ -214,6 +217,9 @@ class CollectionDB : public QObject, public EngineObserver
         void tagsChanged( const MetaBundle &bundle );
 
     public:
+        CollectionDB( bool temporary = false );
+        ~CollectionDB();
+
         static CollectionDB *instance();
 
         QString escapeString( const QString &string ) { return m_dbConnPool->escapeString(string); }
@@ -224,6 +230,7 @@ class CollectionDB : public QObject, public EngineObserver
         // We might consider using LONGTEXT type, as some lyrics could be VERY long..???
         QString longTextColumnType() { if ( m_dbConnPool->getDbConnectionType() == DbConnection::postgresql ) return "TEXT"; else return "TEXT"; }
         QString randomFunc() { if ( m_dbConnPool->getDbConnectionType() == DbConnection::postgresql ) return "random()"; else return "RAND()"; }
+
         int getType() { return m_dbConnPool->getDbConnectionType(); }
 
 
@@ -287,6 +294,7 @@ class CollectionDB : public QObject, public EngineObserver
         int getSongPercentage( const QString &url  );
         int getPlayCount( const QString &url );
         void setSongPercentage( const QString &url , int percentage );
+        void migrateFile( const QString &oldURL, const QString &newURL );
 
         //artist methods
         QStringList similarArtists( const QString &artist, uint count );
@@ -306,7 +314,7 @@ class CollectionDB : public QObject, public EngineObserver
         QStringList albumListOfArtist( const QString &artist, bool withUnknown = true, bool withCompilations = true );
         QStringList artistAlbumList( bool withUnknown = true, bool withCompilations = true );
 
-        QStringList albumTracks( const QString &artist_id, const QString &album_id );
+        QStringList albumTracks( const QString &artist_id, const QString &album_id, const bool isValue = false );
 
         //cover management methods
         /** Returns the image from a given URL, network-transparently.
@@ -341,9 +349,6 @@ class CollectionDB : public QObject, public EngineObserver
         QStringList staleImages();
 
     protected:
-        CollectionDB();
-        ~CollectionDB();
-
         QCString md5sum( const QString& artist, const QString& album, const QString& file = QString::null );
         void engineTrackEnded( int finalPosition, int trackLength );
         /** Manages regular folder monitoring scan */
@@ -396,6 +401,7 @@ class CollectionDB : public QObject, public EngineObserver
 
         DbConnectionPool *m_dbConnPool;
 
+        bool m_isTemporary;
         bool m_monitor;
         QDir m_cacheDir;
         QDir m_coverDir;
@@ -411,7 +417,7 @@ class QueryBuilder
                                tabStats = 64, tabDummy = 0 };
         enum qBuilderOptions { optNoCompilations = 1, optOnlyCompilations = 2, optRemoveDuplicates = 4,
                                optRandomize = 8 };
-        enum qBuilderValues  { valID = 1, valName = 2, valURL = 4, valTitle = 8, valTrack = 16, valScore = 32, 
+        enum qBuilderValues  { valID = 1, valName = 2, valURL = 4, valTitle = 8, valTrack = 16, valScore = 32,
                                valComment = 64, valBitrate = 128, valLength = 256, valSamplerate = 512,
                                valPlayCounter = 1024, valCreateDate = 2048, valAccessDate = 4096,
                                valPercentage = 8192, valArtistID = 16384, valAlbumID = 32768,
@@ -430,6 +436,8 @@ class QueryBuilder
         void beginOR(); //filters will be ORed instead of ANDed
         void endOR();   //don't forget to end it!
 
+        void setGoogleFilter( int defaultTables, QString query );
+
         void addURLFilters( const QStringList& filter );
 
         void addFilter( int tables, const QString& filter);
