@@ -44,7 +44,9 @@ extern "C"
     #include <unistd.h>
 }
 
-#define HELIX_ENGINE_TIMER 10  // 10 ms timer
+#define HELIX_ENGINE_TIMER 10     // 10 ms timer
+#define SCOPE_MAX_BEHIND   200    // 200 postmix buffers
+
 
 
 #ifndef LLONG_MAX
@@ -279,11 +281,11 @@ HelixEngine::load( const KURL &url, bool isStream )
    m_url = url;
 
    if (url.isLocalFile())
-      HelixSimplePlayer::setURL( QFile::encodeName( url.url() ), nextPlayer );
+      HelixSimplePlayer::setURL( QFile::encodeName( url.url() ), nextPlayer, !m_isStream );
    else
    {
       m_isStream = true;
-      HelixSimplePlayer::setURL( QFile::encodeName( url.url() ), nextPlayer );
+      HelixSimplePlayer::setURL( QFile::encodeName( url.url() ), nextPlayer, !m_isStream );
    }
 
    return true;
@@ -474,6 +476,8 @@ HelixEngine::timerEvent( QTimerEvent * )
 
    m_lasttime += HELIX_ENGINE_TIMER;
 
+   prune();
+
 #ifdef DEBUG_PURPOSES_ONLY
    // calculate the frame rate of the scope
    m_ftime += HELIX_ENGINE_TIMER;
@@ -522,24 +526,8 @@ HelixEngine::timerEvent( QTimerEvent * )
 }
 
 
-const Engine::Scope &HelixEngine::scope()
+unsigned long HelixEngine::prune()
 {
-   int i, sb = 0;
-   unsigned long t;
-
-#ifdef DEBUG_PURPOSES_ONLY
-   m_fcount++;
-#endif
-
-   if (!m_inited)
-      return m_scope;
-
-   if (!m_item && !peekScopeTime(t))
-   {
-      m_item = getScopeBuf();
-      if (m_item)
-         sb++;
-   }
    //
    // this bit is to help us keep more accurate time than helix provides
    /////////////////////////////////////////////////////////////////////
@@ -567,12 +555,18 @@ const Engine::Scope &HelixEngine::scope()
    }
    m_lastpos = hpos;
 
+   if ( getScopeCount() > SCOPE_MAX_BEHIND ) // protect against naughty streams
+   {
+      resetScope();
+      return w;
+   }
+
    if (!w || !m_item)
    {
 #ifdef DEBUG_PURPOSES_ONLY
       m_scopebufnone++; // for tuning the scope... (scope is tuned for 44.1kHz sample rate)
 #endif
-      return m_scope;
+      return w;
    }
 
    while (m_item && w > m_item->etime)
@@ -581,13 +575,41 @@ const Engine::Scope &HelixEngine::scope()
       delete m_item;
       m_item = getScopeBuf();
 
-      sb++;
+      m_sb++;
    }
 
+   return w;
+}
+
+const Engine::Scope &HelixEngine::scope()
+{
+   int i;
+   unsigned long t;
+
+   m_sb = 0;
+
 #ifdef DEBUG_PURPOSES_ONLY
-   m_scopebuftotal += sb;
-   if (sb > 1)
-      m_scopebufwaste += (sb-1);
+   m_fcount++;
+#endif
+
+   if (!m_inited)
+      return m_scope;
+
+   if (!m_item && !peekScopeTime(t))
+   {
+      m_item = getScopeBuf();
+      if (m_item)
+         m_sb++;
+   }
+
+   unsigned long w = prune();
+   if (!w)
+      return m_scope;
+
+#ifdef DEBUG_PURPOSES_ONLY
+   m_scopebuftotal += m_sb;
+   if (m_sb > 1)
+      m_scopebufwaste += (m_sb-1);
 #endif
 
    if (!m_item)
@@ -596,9 +618,9 @@ const Engine::Scope &HelixEngine::scope()
       m_scopebufnone++;
 #endif
       return m_scope;
-   }
+   }      
 
-   if (w < m_item->time) // wait for the player to catchup
+   if ( w < m_item->time ) // wait for the player to catchup
       return m_scope;
 
    int j,k=0;
