@@ -240,8 +240,8 @@ void HelixSimplePlayer::cleanUpStream(int /* playerIndex */)
 void HelixSimplePlayer::updateEQgains()
 {
    for (int i = 0; i<nNumPlayers; i++)
-      if (ppctrl[i]->pPostMixHook && isEQenabled())
-         ((HSPPostMixAudioHook *)ppctrl[i]->pPostMixHook)->updateEQgains(m_preamp, m_equalizerGains);
+      if (pFinalAudioHook && isEQenabled())
+         ((HSPFinalAudioHook *)pFinalAudioHook)->updateEQgains(m_preamp, m_equalizerGains);
 }
 
 /*
@@ -496,12 +496,24 @@ void HelixSimplePlayer::init(const char *corelibhome, const char *pluginslibhome
    if (!pPlugin2Handler)
       STDERR("no plugin enumerator\n");
 
+   pAudioHookManager = 0;
+   pFinalAudioHook = 0;
+   pEngine->QueryInterface(IID_IHXAudioHookManager, (void **) &pAudioHookManager);
+   if (!pAudioHookManager)
+      print2stderr("no audio device hook manager\n");
+
+
    // create players
    for (i = 0; i < numPlayers; i++)
    {
       addPlayer();
    }
 
+   // install hook for visualizations, equalizer, and volume control
+   HSPFinalAudioHook *pPMAH = new HSPFinalAudioHook(this);
+   pAudioHookManager->AddHook(pPMAH);
+   pFinalAudioHook = pPMAH;
+
    if (pPlugin2Handler)
    {
       IHXValues* pPluginProps;
@@ -724,15 +736,15 @@ int HelixSimplePlayer::addPlayer()
       ppctrl[nNumPlayers]->pAudioPlayer->SetStreamInfoResponse(pASIR);
       ppctrl[nNumPlayers]->pStreamInfoResponse = pASIR;
 
-      // add the post-mix hook (for the scope and the equalizer)
-      HSPPostMixAudioHook *pPMAH = new HSPPostMixAudioHook(this, nNumPlayers);
-      ppctrl[nNumPlayers]->pAudioPlayer->AddPostMixHook(pPMAH, false /* DisableWrite */, true /* final hook */);
-      ppctrl[nNumPlayers]->pPostMixHook = pPMAH;
-
       // ...and get the CrossFader
       ppctrl[nNumPlayers]->pAudioPlayer->QueryInterface(IID_IHXAudioCrossFade, (void **) &(ppctrl[nNumPlayers]->pCrossFader));
       if (!ppctrl[nNumPlayers]->pCrossFader)
          STDERR("CrossFader not available\n");
+
+      // install hook for visualizations, equalizer, and volume control
+      HSPPostMixAudioHook *pPMAH = new HSPPostMixAudioHook(this, nNumPlayers);
+      ppctrl[nNumPlayers]->pAudioPlayer->AddPostMixHook(pPMAH, false, true);
+      ppctrl[nNumPlayers]->pPostMixHook= pPMAH;
    }
    else
       STDERR("No AudioPlayer Found - how can we play music!!\n");
@@ -746,6 +758,9 @@ int HelixSimplePlayer::addPlayer()
 HelixSimplePlayer::~HelixSimplePlayer()
 {
    tearDown();
+
+   // only delete now, not evertime you re-init the engine, since you need this to re-init the engine
+   delete [] m_device;
 }
 
 void HelixSimplePlayer::tearDown()
@@ -768,9 +783,8 @@ void HelixSimplePlayer::tearDown()
 
       if (ppctrl[i]->pAudioPlayer)
       {
-         
-         ppctrl[i]->pAudioPlayer->RemovePostMixHook((IHXAudioHook *)ppctrl[i]->pPostMixHook);
-         ppctrl[i]->pPostMixHook->Release(); 
+         ppctrl[i]->pAudioPlayer->RemovePostMixHook(ppctrl[i]->pPostMixHook);
+         ppctrl[i]->pPostMixHook->Release();
 
          ppctrl[i]->pAudioPlayer->RemoveStreamInfoResponse((IHXAudioStreamInfoResponse *) ppctrl[i]->pStreamInfoResponse);
 
@@ -815,6 +829,12 @@ void HelixSimplePlayer::tearDown()
       pPluginE->Release();
    if (pPlugin2Handler)
       pPlugin2Handler->Release();
+   if (pAudioHookManager)
+   {
+      pAudioHookManager->RemoveHook(pFinalAudioHook);
+      pFinalAudioHook->Release();
+      pAudioHookManager->Release();
+   }
    if (pEngineContext)
       pEngineContext->Release();
 
@@ -891,8 +911,6 @@ void HelixSimplePlayer::tearDown()
    scopebufhead = 0;
    scopebuftail = 0;
    m_preamp = 0;
-
-   delete [] m_device;
 }
 
 
@@ -1251,8 +1269,12 @@ void HelixSimplePlayer::setDirectHWVolume(int vol)
    }
 }
 
+bool HelixSimplePlayer::isLocal(int playerIndex) const
+{
+   return (ppctrl[playerIndex]->isLocal && duration(playerIndex)); // must have a length to be local
+}
 
-int HelixSimplePlayer::setURL(const char *file, int playerIndex)
+int HelixSimplePlayer::setURL(const char *file, int playerIndex, bool islocal)
 {
    if (playerIndex == ALL_PLAYERS)
    {
@@ -1283,7 +1305,10 @@ int HelixSimplePlayer::setURL(const char *file, int playerIndex)
          
          ppctrl[playerIndex]->pszURL = new char[strlen(pszURLOrig)+strlen(pszAddOn)+1];
          if ( (len + strlen(pszAddOn)) < MAXPATHLEN )
+         {
+            islocal = true;
             sprintf( ppctrl[playerIndex]->pszURL, "%s%s", pszAddOn, pszURLOrig );
+         }
          else
             return -1;
       }
@@ -1296,6 +1321,8 @@ int HelixSimplePlayer::setURL(const char *file, int playerIndex)
             return -1;
       }
 
+      ppctrl[playerIndex]->isLocal = islocal;
+
 #ifdef __NOCROSSFADER__
       STDERR("opening %s on player %d, src cnt %d\n", 
              ppctrl[playerIndex]->pszURL, playerIndex, ppctrl[playerIndex]->pPlayer->GetSourceCount());
@@ -1736,7 +1763,7 @@ void HelixSimplePlayer::setVolume(unsigned long vol, int playerIndex)
          pthread_mutex_lock(&m_engine_m);
 #ifndef HELIX_SW_VOLUME_INTERFACE
          ppctrl[playerIndex]->volume = vol;
-         ((HSPPostMixAudioHook *)ppctrl[playerIndex]->pPostMixHook)->setGain(vol);
+         ((HSPFinalAudioHook *)pFinalAudioHook)->setGain(vol);
 #else
          ppctrl[playerIndex]->pVolume->SetVolume(vol);
 #endif
