@@ -926,6 +926,20 @@ void PlaylistBrowser::scanPodcasts()
     }
 }
 
+
+void PlaylistBrowser::refreshPodcasts( QListViewItem *parent )
+{
+    for( QListViewItem *child = parent->firstChild();
+            child;
+            child = child->nextSibling() )
+    {
+        if( isPodcastChannel( child ) )
+            static_cast<PodcastChannel*>( child )->rescan();
+        else if( isCategory( child ) )
+            refreshPodcasts( child );
+    }
+}
+
 void PlaylistBrowser::addPodcast( QListViewItem *parent )
 {
     bool ok;
@@ -1309,6 +1323,13 @@ bool PlaylistBrowser::deletePlaylists( KURL::List items )
     {
         // TODO We need to check which files have been deleted successfully
         KIO::del( items );
+        // Avoid deleting dirs. See bug #122480
+        for ( KURL::List::iterator it = items.begin(), end = items.end(); it != end; ++it ) {
+            if ( QFileInfo( (*it).path() ).isDir() ) {
+                it = items.remove( it );
+                continue;
+            }
+        }
         return true;
     }
     return false;
@@ -1364,7 +1385,7 @@ bool PlaylistBrowser::createPlaylist( QListViewItem *parent, bool current )
     else
     {
         debug() << "not current!" << endl;
-	
+
 	//Remove any items in Listview that have the same path as this one
         //  Should only happen when overwriting a playlist
         QListViewItem *item = parent->firstChild();
@@ -1778,17 +1799,20 @@ void PlaylistBrowser::savePLS( PlaylistEntry *item, bool append )
     {
         QTextStream stream( &file );
         QPtrList<TrackItemInfo> trackList = append ? item->droppedTracks() : item->trackList();
-        for( TrackItemInfo *info = trackList.first(); info; info = trackList.next() )
+        stream << "NumberOfEntries=" << trackList.count() << endl;
+        int c=1;
+        for( TrackItemInfo *info = trackList.first(); info; info = trackList.next(), ++c )
         {
-            stream << "File=";
+            stream << "File" << c << "=";
             stream << (info->url().protocol() == "file" ? info->url().path() : info->url().url());
-            stream << "\nTitle=";
+            stream << "\nTitle" << c << "=";
             stream << info->title();
-            stream << "\nLength=";
+            stream << "\nLength" << c << "=";
             stream << info->length();
             stream << "\n";
-        }
+         }
 
+        stream << "Version=2\n";
         file.close();
     }
 }
@@ -2326,12 +2350,7 @@ void PlaylistBrowser::showContextMenu( QListViewItem *item, const QPoint &p, int
                 break;
 
             case REFRESH:
-                tracker = item->firstChild();
-                for( ; tracker; tracker = tracker->nextSibling() )
-                {
-                    if( isPodcastChannel( tracker ) )
-                        static_cast<PodcastChannel*>( tracker )->rescan();
-                }
+                refreshPodcasts(item);
                 break;
 
             case CREATE:
