@@ -11,22 +11,27 @@
 #define DEBUG_PREFIX "PlaylistLoader"
 
 #include "amarok.h"
-#include "debug.h"
-#include "playlist.h"
 #include "collectiondb.h"
+#include "debug.h"
 #include "enginecontroller.h"
-#include <kapplication.h>
-#include <kdirlister.h>
-#include <kurl.h>
+#include "playlist.h"
 #include "playlistitem.h"
 #include "playlistloader.h"
+#include "statusbar.h"
+
+#include <qdatetime.h>   //::recurse()
+#include <qeventloop.h>  //::recurse()
 #include <qfile.h>       //::loadPlaylist()
 #include <qlistview.h>
 #include <qregexp.h>
 #include <qstringlist.h>
 #include <qtextstream.h> //::loadPlaylist()
-#include "statusbar.h"
+
 #include <dcopref.h>
+#include <kapplication.h>
+#include <kdirlister.h>
+#include <kurl.h>
+
 
 
 typedef QValueList<QDomNode> NodeList;
@@ -41,6 +46,7 @@ UrlLoader::UrlLoader( const KURL::List &urls, QListViewItem *after, bool playFir
         , m_markerListViewItem( new PlaylistItem( Playlist::instance(), after ) )
         , m_playFirstUrl( playFirstUrl )
         , m_block( "UrlLoader" )
+        , m_oldQueue( Playlist::instance()->m_nextTracks )
 {
     DEBUG_BLOCK
 
@@ -99,9 +105,23 @@ UrlLoader::UrlLoader( const KURL::List &urls, QListViewItem *after, bool playFir
             m_playFirstUrl = false;
         }
 
-        else if( protocol == "fetchcover" )
-            continue;
+        else if( protocol == "fetchcover" ) {
+            // url looks like:   fetchcover:<artist_name> @@@ <album_name>
+           QString myUrl = url.path();
+           if ( myUrl.endsWith( " @@@" ) )
+               myUrl += ' ';
+           const QStringList list = QStringList::split( " @@@ ", myUrl, true );
+           Q_ASSERT( !list.isEmpty() );
+           QString artist_name = list.front();
+           QString album_name  = list.back();
 
+           QStringList trackUrls = CollectionDB::instance()->albumTracks( artist_name, album_name, true );
+           KURL url;
+           foreach( trackUrls ) {
+                url.setPath( *it );
+                m_URLs += url;
+           }
+        }
         else if( protocol == "seek" )
             continue;
 
@@ -141,7 +161,7 @@ UrlLoader::UrlLoader( const KURL::List &urls, QListViewItem *after, bool playFir
         else {
             // this is the best way I found for recursing if required
             // and not recusring if not required
-            KURL::List urls = recurse( url );
+            const KURL::List urls = recurse( url );
 
             // recurse only works on directories, else it swallows the URL
             if( urls.isEmpty() )
@@ -221,7 +241,12 @@ UrlLoader::customEvent( QCustomEvent *e )
     #define e static_cast<TagsEvent*>(e)
     switch( e->type() ) {
     case 1000:
-        foreachType( BundleList, e->bundles ) {
+        foreachType( BundleList, e->bundles )
+        {
+            //Only add files that exist to the playlist
+            if( !(*it).exists() )
+                continue;
+
             PlaylistItem *item = new PlaylistItem( *it, m_markerListViewItem );
 
             if( m_playFirstUrl ) {
@@ -235,6 +260,7 @@ UrlLoader::customEvent( QCustomEvent *e )
     {
         const QString QUEUE_INDEX = "queue_index";
         const QString STOP_AFTER = "stop_after";
+        const QString DISABLED = "disabled";
         foreachType( NodeList, e->nodes )
         {
             if( (*it).isNull() ) //safety
@@ -266,6 +292,9 @@ UrlLoader::customEvent( QCustomEvent *e )
             }
             if( element.hasAttribute( STOP_AFTER ) )
                 Playlist::instance()->m_stopAfterTrack = item;
+
+            if( element.hasAttribute( DISABLED ) )
+                item->setEnabled( false );
         }
         break;
     }
@@ -280,6 +309,16 @@ UrlLoader::customEvent( QCustomEvent *e )
 void
 UrlLoader::completeJob()
 {
+    const PLItemList &newQueue = Playlist::instance()->m_nextTracks;
+    QPtrListIterator<PlaylistItem> it( newQueue );
+    PLItemList added;
+    for( it.toFirst(); *it; ++it )
+        if( !m_oldQueue.containsRef( *it ) )
+            added << (*it);
+
+    if( !added.isEmpty() )
+        emit queueChanged( added, PLItemList() );
+
     if ( !m_badURLs.isEmpty() ) {
         amaroK::StatusBar::instance()->shortLongMessage(
                 i18n("Some media could not be loaded (not playable)."),
@@ -290,15 +329,10 @@ UrlLoader::completeJob()
             debug() << "\t" << m_badURLs[it] << endl;
     }
 
-    // delete now or the playlist count in the statusbar will be wrong
-    delete m_markerListViewItem;
-    m_markerListViewItem = 0;
-
     //syncronous, ie not using eventLoop
     QApplication::sendEvent( dependent(), this );
 }
 
-#include <qeventloop.h>
 KURL::List
 UrlLoader::recurse( const KURL &url )
 {
@@ -307,9 +341,15 @@ UrlLoader::recurse( const KURL &url )
     KDirLister lister( false );
     lister.setAutoUpdate( false );
     lister.setAutoErrorHandlingEnabled( false, 0 );
-    lister.openURL( url );
+    if ( !lister.openURL( url ) )
+        return KURL::List();
 
-    while( !lister.isFinished() && !isAborted() )
+    // Fucking KDirLister sometimes hangs on remote media, so we add a timeout
+    const int timeout = 3000; // ms
+    QTime watchdog;
+    watchdog.start();
+
+    while( !lister.isFinished() && !isAborted() && watchdog.elapsed() < timeout )
         kapp->eventLoop()->processEvents( QEventLoop::ExcludeUserInput );
 
     KFileItemList items = lister.items(); //returns QPtrList, so we MUST only do it once!
@@ -343,41 +383,34 @@ UrlLoader::loadXml( const KURL &url )
     stream.setEncoding( QTextStream::UnicodeUTF8 );
 
     QDomDocument d;
-    if( !d.setContent( stream.read() ) ) {
+    QString er;
+    int l, c;
+    if( !d.setContent( stream.read(), &er, &l, &c ) ) { // return error values
         amaroK::StatusBar::instance()->longMessageThreadSafe( i18n(
                 //TODO add a link to the path to the playlist
                 "The XML in the playlist was invalid. Please report this as a bug to the amaroK "
-                "developers. Thank you." ) );
+                "developers. Thank you." ), KDE::StatusBar::Error );
+        error() << "[PLAYLISTLOADER]: Error loading xml file: " << url.prettyURL() << "(" << er << ")"
+                << " at line " << l << ", column " << c << endl;
         return;
     }
 
-    const PLItemList oldQueue = Playlist::instance()->m_nextTracks;
     NodeList nodes;
-    TagsEvent e;
+    TagsEvent* e = new TagsEvent();
     const QString ITEM( "item" ); //so we don't construct this QString all the time
     for( QDomNode n = d.namedItem( "playlist" ).firstChild(); !n.isNull(); n = n.nextSibling() )
     {
         if( n.nodeName() != ITEM ) continue;
 
         if( !n.toElement().isNull() )
-            e.nodes += n;
+            e->nodes += n;
 
-        if( e.nodes.count() == OPTIMUM_BUNDLE_COUNT ) {
-            QApplication::sendEvent( this, &e );
-            e = TagsEvent();
+        if( e->nodes.count() == OPTIMUM_BUNDLE_COUNT ) {
+            QApplication::postEvent( this, e );
+            e = new TagsEvent();
         }
     }
-    QApplication::sendEvent( this, &e );
-
-    const PLItemList &newQueue = Playlist::instance()->m_nextTracks;
-    QPtrListIterator<PlaylistItem> it( newQueue );
-    PLItemList added;
-    for( it.toFirst(); *it; ++it )
-        if( !oldQueue.containsRef( *it ) )
-            added << (*it);
-
-    if( !added.isEmpty() )
-        emit queueChanged( added, PLItemList() );
+    QApplication::postEvent( this, e );
 }
 
 
