@@ -175,6 +175,8 @@ ScriptManager::~ScriptManager()
     ScriptMap::Iterator end( m_scripts.end() );
     for( it = m_scripts.begin(); it != end; ++it ) {
         if( it.data().process ) {
+            it.data().process->kill(); // Sends SIGTERM
+            it.data().process->detach();
             delete it.data().process;
             runningScripts << it.key();
         }
@@ -463,12 +465,13 @@ ScriptManager::slotRunScript()
     if( m_scripts[name].process ) return false;
 
     AmaroKProcIO* script = new AmaroKProcIO();
-    script->setComm( (KProcess::Communication) ( KProcess::Stdin|KProcess::Stderr ) );
+    script->setComm( (KProcess::Communication) ( KProcess::All ) );
     const KURL url = m_scripts[name].url;
     *script << url.path();
     script->setWorkingDirectory( amaroK::saveLocation( "scripts-data/" ) );
 
     connect( script, SIGNAL( receivedStderr( KProcess*, char*, int ) ), SLOT( slotReceivedStderr( KProcess*, char*, int ) ) );
+    connect( script, SIGNAL( receivedStdout( KProcess*, char*, int ) ), SLOT( slotReceivedStdout( KProcess*, char*, int ) ) );
     connect( script, SIGNAL( processExited( KProcess* ) ), SLOT( scriptFinished( KProcess* ) ) );
 
     if( !script->start( KProcess::NotifyOnExit ) ) {
@@ -609,6 +612,13 @@ ScriptManager::slotShowContextMenu( QListViewItem* item, const QPoint& pos )
     }
 }
 
+/* This is just a workaround, some scripts crash for some people if stdout is not handled.
+   Anyway, printing out the output of scripts can be usefull. */
+void
+ScriptManager::slotReceivedStdout( KProcess* /*process*/, char* buf, int len )
+{
+    debug() << QString::fromLatin1( buf, len ) << endl;
+}
 
 void
 ScriptManager::slotReceivedStderr( KProcess* process, char* buf, int len )
