@@ -47,7 +47,7 @@ class KProcIO;
  *   trackChange
  *   volumeChange: newVolume (range: 0-100)
  *
- * @see http://amarok.kde.org/wiki/index.php/Script-Writing_HowTo
+ * @see http://amarok.kde.org/amarokwiki/index.php/Script-Writing_HowTo
  */
 
 class ScriptManager : public KDialogBase, public EngineObserver
@@ -98,6 +98,7 @@ class ScriptManager : public KDialogBase, public EngineObserver
         void slotAboutScript();
         void slotShowContextMenu( QListViewItem*, const QPoint& );
 
+        void slotReceivedStderr( KProcess*, char*, int );
         void slotReceivedStdout( KProcess*, char*, int );
         void scriptFinished( KProcess* process );
 
