@@ -226,9 +226,9 @@ Vis::Selector::processExited( KProcess *proc )
 
 // Shouldn't be necessary, but it's part of a fix to make libvisual work again when running with amarok binary
 void
-Vis::Selector::receivedStdout( KProcess */*proc*/, char* buffer, int /*length*/ )
+Vis::Selector::receivedStdout( KProcess */*proc*/, char* buffer, int length )
 {
-     debug() << buffer;
+     debug() << QString::fromLatin1( buffer, length ) << endl;
 }
 
 void
