@@ -32,6 +32,8 @@
 #include <kstdguiitem.h>
 
 #include <qapplication.h>
+#include <qdatetime.h>      //writeLogFile()
+#include <qfile.h>          //writeLogFile()
 #include <qpushbutton.h>
 #include <qlabel.h>
 #include <qlayout.h>
@@ -83,6 +85,7 @@ namespace SingleShotPool
 
 StatusBar::StatusBar( QWidget *parent, const char *name )
         : QWidget( parent, name )
+        , m_logCounter( -1 )
 {
     QBoxLayout *mainlayout = new QHBoxLayout( this, 2, /*spacing*/5 );
 
@@ -116,12 +119,11 @@ StatusBar::StatusBar( QWidget *parent, const char *name )
     connect( b1, SIGNAL(clicked()), SLOT(abortAllProgressOperations()) );
     connect( b2, SIGNAL(toggled( bool )), SLOT(toggleProgressWindow( bool )) );
 
-    m_popupProgress = new PopupMessage( this, mainProgressBarBox, 0, "popupProgress" );
-    m_popupProgress->showCloseButton( false );
-    m_popupProgress->showCounter( false );
+    m_popupProgress = new OverlayWidget( this, mainProgressBarBox, "popupProgress" );
     m_popupProgress->setMargin( 1 );
     m_popupProgress->setFrameStyle( QFrame::Box | QFrame::Raised );
     m_popupProgress->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
+   (new QGridLayout( m_popupProgress, 1 /*rows*/, 3 /*cols*/, 6, 3 ))->setAutoAdd( true );
 }
 
 void
@@ -212,6 +214,8 @@ StatusBar::shortMessage( const QString &text )
     m_mainTextLabel->setPalette( QToolTip::palette() );
 
     SingleShotPool::startTimer( 5000, this, SLOT(resetMainText()) );
+
+    writeLogFile( text );
 }
 
 void
@@ -258,25 +262,47 @@ StatusBar::shortLongMessage( const QString &_short, const QString &_long )
 }
 
 void
-StatusBar::longMessage( const QString &text, int /*type*/ )
+StatusBar::longMessage( const QString &text, int type )
 {
+    if( text.isEmpty() )
+        return;
+
     PopupMessage * message;
     message = new PopupMessage( this, m_mainTextLabel );
     message->setText( text );
 
-    if ( m_messageQueue.isEmpty() )
+    QString image;
+
+    switch( type )
     {
-        message->move( 0, y() - message->height() );
-        message->display();
-    } else {
-        message->stackUnder( m_messageQueue.last() );
-        message->display();
-        message->move( 0, y() - message->height() );
+        case Information:
+        case Question:
+            image = KGlobal::iconLoader()->iconPath( "messagebox_info", -KIcon::SizeHuge );
+            break;
+
+        case Sorry:
+        case Warning:
+            image = KGlobal::iconLoader()->iconPath( "messagebox_warning", -KIcon::SizeHuge );
+            break;
+
+        case Error:
+            image = KGlobal::iconLoader()->iconPath( "messagebox_critical", -KIcon::SizeHuge );
+            break;
     }
+    if( !image.isEmpty() )
+        message->setImage( image );
+
+    if ( !m_messageQueue.isEmpty() )
+         message->stackUnder( m_messageQueue.last() );
+
+    message->reposition();
+    message->display();
 
     raise();
 
     m_messageQueue += message;
+
+    writeLogFile( text );
 }
 
 void
@@ -321,9 +347,7 @@ StatusBar::newProgressOperation( QObject *owner )
     else
         static_cast<QWidget*>(progressBox()->child("showAllProgressDetails"))->show();
     QLabel *label = new QLabel( m_popupProgress );
-    ProgressBar *pBar = new ProgressBar( m_popupProgress, label );
-    m_popupProgress->addWidget( pBar );
-    m_progressMap.insert( owner, pBar );
+    m_progressMap.insert( owner, new ProgressBar( m_popupProgress, label ) );
 
 
     connect( owner, SIGNAL(destroyed( QObject* )), SLOT(endProgressOperation( QObject* )) );
@@ -427,7 +451,7 @@ StatusBar::hideMainProgressBar()
         resetMainText();
 
         m_mainProgressBar->setProgress( 0 );
-        progressBox()->close();
+        progressBox()->hide();
     }
 }
 
@@ -539,6 +563,53 @@ StatusBar::pruneProgressBars()
     }
 }
 
+/// Method which writes to a rotating log file.
+void
+StatusBar::writeLogFile( const QString &text )
+{
+    if( text.isEmpty() ) return;
+
+    const int counter = 4; // number of logs to keep
+    int c = counter;
+    QString logBase = amaroK::saveLocation() + "statusbar.log.";
+    QFile file;
+
+    if( m_logCounter < 0 ) //find which log to write to
+    {
+        for( ; c > 0; c-- )
+        {
+            QString log = logBase + QString::number(c);
+
+            if( QFile::exists( log ) )
+            {
+                file.setName( log );
+            }
+        }
+        if( c == 0 ) file.setName( logBase + "0" );
+        m_logCounter = c;
+    }
+    else
+    {
+        file.setName( logBase + QString::number(m_logCounter) );
+    }
+
+    if( file.size() > 30000 ) // approximately 1000 lines per log file
+    {
+        m_logCounter++;
+        m_logCounter = m_logCounter % counter;
+
+        file.setName( logBase + QString::number(m_logCounter) );
+        // if we have overflown the log, then we want to overwrite the previous content
+        if( !file.open( IO_WriteOnly ) ) return;
+    }
+    else if( !file.open( IO_WriteOnly|IO_Append ) ) return;
+
+    QTextStream stream( &file );
+    stream.setEncoding( QTextStream::UnicodeUTF8 );
+
+    stream << "[" << QDateTime::currentDateTime().toString(Qt::LocalDate) << "] " << text << endl;
+}
+
 } //namespace KDE
 
 
