@@ -168,7 +168,7 @@ TagDialog::musicbrainzQuery() //SLOT
 
     m_mbTrack = m_bundle.url().path();
     KTRMLookup* ktrm = new KTRMLookup( m_mbTrack, true );
-    connect( ktrm, SIGNAL( sigResult( KTRMResultList ) ), SLOT( queryDone( KTRMResultList ) ) );
+    connect( ktrm, SIGNAL( sigResult( KTRMResultList, QString ) ), SLOT( queryDone( KTRMResultList, QString ) ) );
 
     pushButton_musicbrainz->setEnabled( false );
     pushButton_musicbrainz->setText( i18n( "Generating audio fingerprint..." ) );
@@ -177,17 +177,22 @@ TagDialog::musicbrainzQuery() //SLOT
 }
 
 void
-TagDialog::queryDone( KTRMResultList results ) //SLOT
+TagDialog::queryDone( KTRMResultList results, QString error ) //SLOT
 {
 #if HAVE_TUNEPIMP
 
-    if ( !results.isEmpty() )
-    {
-        TrackPickerDialog* t = new TrackPickerDialog(m_bundle.url().filename(),results,this);
-        t->show();
+    if ( !error.isEmpty() ) {
+        KMessageBox::sorry( this, error );
+    }
+    else {
+        if ( !results.isEmpty() )
+        {
+            TrackPickerDialog* t = new TrackPickerDialog(m_bundle.url().filename(),results,this);
+            t->show();
+        }
+        else
+            KMessageBox::sorry( this, i18n( "The track was not found in the MusicBrainz database." ) );
     }
-    else
-        KMessageBox::sorry( this, i18n( "The track was not found in the MusicBrainz database." ) );
 
     QApplication::restoreOverrideCursor();
     pushButton_musicbrainz->setEnabled( true );
